# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: upload.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import locker_pb2 as locker__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='upload.proto',
  package='wireless_android_skyjam',
  syntax='proto2',
  serialized_pb=_b('\n\x0cupload.proto\x12\x17wireless_android_skyjam\x1a\x0clocker.proto\"\xb1\x01\n\x0eResponseStatus\x12K\n\rresponse_code\x18\x01 \x02(\x0e\x32\x34.wireless_android_skyjam.ResponseStatus.ResponseCode\"R\n\x0cResponseCode\x12\x06\n\x02OK\x10\x01\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x02\x12\x0e\n\nSOFT_ERROR\x10\x03\x12\x16\n\x12METADATA_TOO_LARGE\x10\x04\"\xa7\x01\n\x0fUploadOperation\x12\x45\n\toperation\x18\x01 \x02(\x0e\x32\x32.wireless_android_skyjam.UploadOperation.Operation\"M\n\tOperation\x12\x14\n\x10OPERATION_CREATE\x10\x01\x12\x14\n\x10OPERATION_MODIFY\x10\x02\x12\x14\n\x10OPERATION_DELETE\x10\x03\"\xa1\x01\n\x11\x45nhancedChallenge\x12P\n\x0ehash_algorithm\x18\x01 \x01(\x0e\x32\x38.wireless_android_skyjam.EnhancedChallenge.HashAlgorithm\x12\x13\n\x0brandom_seed\x18\x02 \x01(\x0c\"%\n\rHashAlgorithm\x12\x0b\n\x07SHA_256\x10\x01\x12\x07\n\x03MD5\x10\x02\"\xac\x01\n\x11\x43lientFpChallenge\x12^\n\x15\x66ingerprint_algorithm\x18\x01 \x01(\x0e\x32?.wireless_android_skyjam.ClientFpChallenge.FingerprintAlgorithm\"7\n\x14\x46ingerprintAlgorithm\x12\x08\n\x04NONE\x10\x00\x12\x15\n\x11LIGHTMP3FP_CLIENT\x10\x01\"\xca\x02\n\rChallengeInfo\x12\x17\n\x0f\x63lient_track_id\x18\x01 \x02(\t\x12\x14\n\x0cstart_millis\x18\x02 \x02(\x05\x12\x17\n\x0f\x64uration_millis\x18\x03 \x02(\x05\x12\x19\n\x11\x63hallenge_user_id\x18\x04 \x01(\t\x12\x1b\n\x13\x63hallenge_timestamp\x18\x05 \x01(\x03\x12\x14\n\x0c\x65xpect_match\x18\x06 \x01(\x08\x12\x46\n\x12\x65nhanced_challenge\x18\x07 \x01(\x0b\x32*.wireless_android_skyjam.EnhancedChallenge\x12\x12\n\ncandidates\x18\x08 \x03(\t\x12G\n\x13\x63lient_fp_challenge\x18\t \x01(\x0b\x32*.wireless_android_skyjam.ClientFpChallenge\"5\n\x19\x45nhancedChallengeResponse\x12\x18\n\x10\x63hallenge_answer\x18\x01 \x01(\x0c\"h\n\x13SignedChallengeInfo\x12>\n\x0e\x63hallenge_info\x18\x01 \x02(\x0b\x32&.wireless_android_skyjam.ChallengeInfo\x12\x11\n\tsignature\x18\x02 \x02(\x0c\"\xbe\x02\n\x0bTrackSample\x12-\n\x05track\x18\x01 \x02(\x0b\x32\x1e.wireless_android_skyjam.Track\x12K\n\x15signed_challenge_info\x18\x02 \x02(\x0b\x32,.wireless_android_skyjam.SignedChallengeInfo\x12\x0e\n\x06sample\x18\x03 \x01(\x0c\x12\x41\n\rsample_format\x18\x04 \x01(\x0e\x32*.wireless_android_skyjam.Track.ContentType\x12;\n\x0euser_album_art\x18\x05 \x01(\x0b\x32#.wireless_android_skyjam.ImageUnion\x12#\n\x1b\x65nhanced_challenge_response\x18\x06 \x01(\x0c\"\xa5\x01\n\x15UploadPlaylistRequest\x12\x42\n\x10upload_operation\x18\x01 \x02(\x0b\x32(.wireless_android_skyjam.UploadOperation\x12\x33\n\x08playlist\x18\x02 \x03(\x0b\x32!.wireless_android_skyjam.Playlist\x12\x13\n\x0buploader_id\x18\x03 \x02(\t\"z\n\x10PlaylistResponse\x12@\n\x0fresponse_status\x18\x01 \x02(\x0b\x32\'.wireless_android_skyjam.ResponseStatus\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x11\n\tserver_id\x18\x03 \x01(\t\"^\n\x16UploadPlaylistResponse\x12\x44\n\x11playlist_response\x18\x01 \x03(\x0b\x32).wireless_android_skyjam.PlaylistResponse\"\xb5\x01\n\x1aUploadPlaylistEntryRequest\x12\x42\n\x10upload_operation\x18\x01 \x02(\x0b\x32(.wireless_android_skyjam.UploadOperation\x12>\n\x0eplaylist_entry\x18\x02 \x03(\x0b\x32&.wireless_android_skyjam.PlaylistEntry\x12\x13\n\x0buploader_id\x18\x03 \x02(\t\"\x7f\n\x15PlaylistEntryResponse\x12@\n\x0fresponse_status\x18\x01 \x02(\x0b\x32\'.wireless_android_skyjam.ResponseStatus\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x11\n\tserver_id\x18\x03 \x01(\t\"n\n\x1bUploadPlaylistEntryResponse\x12O\n\x17playlist_entry_response\x18\x01 \x03(\x0b\x32..wireless_android_skyjam.PlaylistEntryResponse\"[\n\x15UploadMetadataRequest\x12-\n\x05track\x18\x01 \x03(\x0b\x32\x1e.wireless_android_skyjam.Track\x12\x13\n\x0buploader_id\x18\x02 \x02(\t\"\xb0\x01\n\x18UpdateUploadStateRequest\x12L\n\x05state\x18\x01 \x02(\x0e\x32=.wireless_android_skyjam.UpdateUploadStateRequest.UploadState\x12\x13\n\x0buploader_id\x18\x02 \x02(\t\"1\n\x0bUploadState\x12\t\n\x05START\x10\x01\x12\n\n\x06PAUSED\x10\x02\x12\x0b\n\x07STOPPED\x10\x03\"M\n\x12\x43lientStateRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\x12\"\n\x1aget_purchased_tracks_since\x18\x02 \x01(\x03\"\x80\x02\n\x13\x43lientStateResponse\x12\x1a\n\x12locker_track_limit\x18\x01 \x01(\x03\x12\x1c\n\x14user_songs_in_locker\x18\x02 \x01(\x03\x12\x1e\n\x16track_size_limit_in_mb\x18\x03 \x01(\x05\x12#\n\x1buser_purchased_tracks_since\x18\x04 \x01(\x03\x12\x19\n\x11total_track_count\x18\x05 \x01(\x03\x12O\n\x0foverride_config\x18\x06 \x01(\x0b\x32\x36.wireless_android_skyjam.OverrideConfigValueCollection\"\xc5\x01\n\x16UploadMetadataResponse\x12K\n\x15signed_challenge_info\x18\x01 \x03(\x0b\x32,.wireless_android_skyjam.SignedChallengeInfo\x12\x11\n\tupload_id\x18\x02 \x03(\t\x12K\n\x15track_sample_response\x18\x03 \x03(\x0b\x32,.wireless_android_skyjam.TrackSampleResponse\"\xbd\x03\n\x13TrackSampleResponse\x12\x17\n\x0f\x63lient_track_id\x18\x01 \x02(\t\x12P\n\rresponse_code\x18\x02 \x02(\x0e\x32\x39.wireless_android_skyjam.TrackSampleResponse.ResponseCode\x12\x17\n\x0fserver_track_id\x18\x03 \x01(\t\x12\x15\n\ralbum_art_url\x18\x04 \x01(\t\x12\x14\n\x0c\x65xpect_match\x18\x05 \x01(\x08\"\xf4\x01\n\x0cResponseCode\x12\x0b\n\x07MATCHED\x10\x01\x12\x14\n\x10UPLOAD_REQUESTED\x10\x02\x12\x15\n\x11INVALID_SIGNATURE\x10\x03\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x04\x12\x13\n\x0fTRANSIENT_ERROR\x10\x05\x12\x13\n\x0fPERMANENT_ERROR\x10\x06\x12\x1d\n\x19TRACK_COUNT_LIMIT_REACHED\x10\x07\x12\x16\n\x12REJECT_STORE_TRACK\x10\x08\x12\x1f\n\x1bREJECT_STORE_TRACK_BY_LABEL\x10\t\x12\x14\n\x10REJECT_DRM_TRACK\x10\n\"f\n\x13UploadSampleRequest\x12:\n\x0ctrack_sample\x18\x01 \x03(\x0b\x32$.wireless_android_skyjam.TrackSample\x12\x13\n\x0buploader_id\x18\x02 \x02(\t\"c\n\x14UploadSampleResponse\x12K\n\x15track_sample_response\x18\x01 \x03(\x0b\x32,.wireless_android_skyjam.TrackSampleResponse\";\n\nImageUnion\x12\x16\n\x0euser_album_art\x18\x01 \x01(\x0c\x12\x15\n\ralbum_art_url\x18\x02 \x01(\t\"\xe4\t\n\x0eUploadResponse\x12K\n\rresponse_type\x18\x01 \x01(\x0e\x32\x34.wireless_android_skyjam.UploadResponse.ResponseType\x12J\n\x11metadata_response\x18\x02 \x01(\x0b\x32/.wireless_android_skyjam.UploadMetadataResponse\x12J\n\x11playlist_response\x18\x03 \x01(\x0b\x32/.wireless_android_skyjam.UploadPlaylistResponse\x12U\n\x17playlist_entry_response\x18\x04 \x01(\x0b\x32\x34.wireless_android_skyjam.UploadPlaylistEntryResponse\x12\x46\n\x0fsample_response\x18\x05 \x01(\x0b\x32-.wireless_android_skyjam.UploadSampleResponse\x12\x42\n\x10getjobs_response\x18\x07 \x01(\x0b\x32(.wireless_android_skyjam.GetJobsResponse\x12J\n\x14\x63lientstate_response\x18\x08 \x01(\x0b\x32,.wireless_android_skyjam.ClientStateResponse\x12\x35\n\x06policy\x18\x06 \x01(\x0b\x32%.wireless_android_skyjam.ClientPolicy\x12G\n\x0b\x61uth_status\x18\x0b \x01(\x0e\x32\x32.wireless_android_skyjam.UploadResponse.AuthStatus\x12\x12\n\nauth_error\x18\x0c \x01(\x08\x12@\n\x0fupauth_response\x18\r \x01(\x0b\x32\'.wireless_android_skyjam.UpAuthResponse\"\xfa\x01\n\x0cResponseType\x12\x15\n\x11METADATA_RESPONSE\x10\x01\x12\x15\n\x11PLAYLIST_RESPONSE\x10\x02\x12\x1b\n\x17PLAYLIST_ENTRY_RESPONSE\x10\x03\x12\x13\n\x0fSAMPLE_RESPONSE\x10\x04\x12\x14\n\x10GETJOBS_RESPONSE\x10\x05\x12\x11\n\rAUTH_RESPONSE\x10\x06\x12\x19\n\x15\x43LIENT_STATE_RESPONSE\x10\x07\x12 \n\x1cUPDATE_UPLOAD_STATE_RESPONSE\x10\x08\x12$\n DELETE_UPLOAD_REQUESTED_RESPONSE\x10\t\"\xea\x01\n\nAuthStatus\x12\x06\n\x02OK\x10\x08\x12\x15\n\x11MAX_LIMIT_REACHED\x10\t\x12!\n\x1d\x43LIENT_BOUND_TO_OTHER_ACCOUNT\x10\n\x12\x19\n\x15\x43LIENT_NOT_AUTHORIZED\x10\x0b\x12\"\n\x1eMAX_PER_MACHINE_USERS_EXCEEDED\x10\x0c\x12\x17\n\x13\x43LIENT_PLEASE_RETRY\x10\r\x12\x12\n\x0eNOT_SUBSCRIBED\x10\x0e\x12\x13\n\x0fINVALID_REQUEST\x10\x0f\x12\x19\n\x15UPGRADE_MUSIC_MANAGER\x10\x10\"\xb4\x01\n\x0eTracksToUpload\x12\x11\n\tclient_id\x18\x01 \x02(\t\x12\x11\n\tserver_id\x18\x02 \x02(\t\x12\x43\n\x06status\x18\x05 \x02(\x0e\x32\x33.wireless_android_skyjam.TracksToUpload.TrackStatus\"7\n\x0bTrackStatus\x12\x12\n\x0e\x46ORCE_REUPLOAD\x10\x03\x12\x14\n\x10UPLOAD_REQUESTED\x10\x04\"%\n\x0eGetJobsRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\"p\n\x0fGetJobsResponse\x12\x41\n\x10tracks_to_upload\x18\x01 \x03(\x0b\x32\'.wireless_android_skyjam.TracksToUpload\x12\x1a\n\x12get_tracks_success\x18\x02 \x02(\x08\"l\n\rUpAuthRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\x12\x15\n\rfriendly_name\x18\x02 \x01(\t\x12\x16\n\x0emac_identifier\x18\x03 \x01(\t\x12\x17\n\x0fhash_identifier\x18\x04 \x01(\t\"%\n\x0eUpAuthResponse\x12\x13\n\x0buploader_id\x18\x01 \x01(\t\"3\n\x1c\x44\x65leteUploadRequestedRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\"\x83\x02\n\x0c\x43lientPolicy\x12\x15\n\rpause_uploads\x18\x01 \x01(\x08\x12\r\n\x05\x61\x62ort\x18\x02 \x01(\x08\x12\x18\n\x10retry_in_minutes\x18\x03 \x01(\x05\x12\x1a\n\x12\x62\x61ndwidth_cap_kbps\x18\x04 \x01(\x05\x12\x17\n\x0fpause_downloads\x18\x05 \x01(\x08\x12#\n\x1b\x64ownload_bandwidth_cap_kbps\x18\x06 \x01(\x05\x12\x16\n\x0esend_analytics\x18\x07 \x01(\x08\x12\x15\n\ranalytics_url\x18\x08 \x01(\t\x12\x16\n\x0e\x65nable_gapless\x18\t \x01(\x08\x12\x12\n\nenable_m4p\x18\n \x01(\x08\"p\n\x1dUploadMetadataInternalRequest\x12\x0e\n\x06gaiaid\x18\x01 \x02(\x04\x12?\n\x07request\x18\x02 \x02(\x0b\x32..wireless_android_skyjam.UploadMetadataRequest\"\x9f\x02\n\x13OverrideConfigValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12O\n\x08priority\x18\x03 \x01(\x0e\x32=.wireless_android_skyjam.OverrideConfigValue.OverridePriority\"\x9a\x01\n\x10OverridePriority\x12\x1d\n\x10\x44\x45\x46\x41ULT_PRIORITY\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x13\n\x0fLOWEST_PRIORITY\x10\x64\x12\x11\n\x0cLOW_PRIORITY\x10\xc8\x01\x12\x14\n\x0fMEDIUM_PRIORITY\x10\xac\x02\x12\x12\n\rHIGH_PRIORITY\x10\x90\x03\x12\x15\n\x10HIGHEST_PRIORITY\x10\xf4\x03\"\\\n\x1dOverrideConfigValueCollection\x12;\n\x05value\x18\x01 \x03(\x0b\x32,.wireless_android_skyjam.OverrideConfigValue')
  ,
  dependencies=[locker__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_RESPONSESTATUS_RESPONSECODE = _descriptor.EnumDescriptor(
  name='ResponseCode',
  full_name='wireless_android_skyjam.ResponseStatus.ResponseCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SOFT_ERROR', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='METADATA_TOO_LARGE', index=3, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=151,
  serialized_end=233,
)
_sym_db.RegisterEnumDescriptor(_RESPONSESTATUS_RESPONSECODE)

_UPLOADOPERATION_OPERATION = _descriptor.EnumDescriptor(
  name='Operation',
  full_name='wireless_android_skyjam.UploadOperation.Operation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OPERATION_CREATE', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATION_MODIFY', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATION_DELETE', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=326,
  serialized_end=403,
)
_sym_db.RegisterEnumDescriptor(_UPLOADOPERATION_OPERATION)

_ENHANCEDCHALLENGE_HASHALGORITHM = _descriptor.EnumDescriptor(
  name='HashAlgorithm',
  full_name='wireless_android_skyjam.EnhancedChallenge.HashAlgorithm',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SHA_256', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MD5', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=530,
  serialized_end=567,
)
_sym_db.RegisterEnumDescriptor(_ENHANCEDCHALLENGE_HASHALGORITHM)

_CLIENTFPCHALLENGE_FINGERPRINTALGORITHM = _descriptor.EnumDescriptor(
  name='FingerprintAlgorithm',
  full_name='wireless_android_skyjam.ClientFpChallenge.FingerprintAlgorithm',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LIGHTMP3FP_CLIENT', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=687,
  serialized_end=742,
)
_sym_db.RegisterEnumDescriptor(_CLIENTFPCHALLENGE_FINGERPRINTALGORITHM)

_UPDATEUPLOADSTATEREQUEST_UPLOADSTATE = _descriptor.EnumDescriptor(
  name='UploadState',
  full_name='wireless_android_skyjam.UpdateUploadStateRequest.UploadState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='START', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PAUSED', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2593,
  serialized_end=2642,
)
_sym_db.RegisterEnumDescriptor(_UPDATEUPLOADSTATEREQUEST_UPLOADSTATE)

_TRACKSAMPLERESPONSE_RESPONSECODE = _descriptor.EnumDescriptor(
  name='ResponseCode',
  full_name='wireless_android_skyjam.TrackSampleResponse.ResponseCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MATCHED', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPLOAD_REQUESTED', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_SIGNATURE', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRANSIENT_ERROR', index=4, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERMANENT_ERROR', index=5, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRACK_COUNT_LIMIT_REACHED', index=6, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REJECT_STORE_TRACK', index=7, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REJECT_STORE_TRACK_BY_LABEL', index=8, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REJECT_DRM_TRACK', index=9, number=10,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3384,
  serialized_end=3628,
)
_sym_db.RegisterEnumDescriptor(_TRACKSAMPLERESPONSE_RESPONSECODE)

_UPLOADRESPONSE_RESPONSETYPE = _descriptor.EnumDescriptor(
  name='ResponseType',
  full_name='wireless_android_skyjam.UploadResponse.ResponseType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='METADATA_RESPONSE', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PLAYLIST_RESPONSE', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PLAYLIST_ENTRY_RESPONSE', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SAMPLE_RESPONSE', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GETJOBS_RESPONSE', index=4, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AUTH_RESPONSE', index=5, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLIENT_STATE_RESPONSE', index=6, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPDATE_UPLOAD_STATE_RESPONSE', index=7, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DELETE_UPLOAD_REQUESTED_RESPONSE', index=8, number=9,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=4662,
  serialized_end=4912,
)
_sym_db.RegisterEnumDescriptor(_UPLOADRESPONSE_RESPONSETYPE)

_UPLOADRESPONSE_AUTHSTATUS = _descriptor.EnumDescriptor(
  name='AuthStatus',
  full_name='wireless_android_skyjam.UploadResponse.AuthStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MAX_LIMIT_REACHED', index=1, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLIENT_BOUND_TO_OTHER_ACCOUNT', index=2, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLIENT_NOT_AUTHORIZED', index=3, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MAX_PER_MACHINE_USERS_EXCEEDED', index=4, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLIENT_PLEASE_RETRY', index=5, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_SUBSCRIBED', index=6, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_REQUEST', index=7, number=15,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPGRADE_MUSIC_MANAGER', index=8, number=16,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=4915,
  serialized_end=5149,
)
_sym_db.RegisterEnumDescriptor(_UPLOADRESPONSE_AUTHSTATUS)

_TRACKSTOUPLOAD_TRACKSTATUS = _descriptor.EnumDescriptor(
  name='TrackStatus',
  full_name='wireless_android_skyjam.TracksToUpload.TrackStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FORCE_REUPLOAD', index=0, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPLOAD_REQUESTED', index=1, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=5277,
  serialized_end=5332,
)
_sym_db.RegisterEnumDescriptor(_TRACKSTOUPLOAD_TRACKSTATUS)

_OVERRIDECONFIGVALUE_OVERRIDEPRIORITY = _descriptor.EnumDescriptor(
  name='OverridePriority',
  full_name='wireless_android_skyjam.OverrideConfigValue.OverridePriority',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEFAULT_PRIORITY', index=0, number=-1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOWEST_PRIORITY', index=1, number=100,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOW_PRIORITY', index=2, number=200,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MEDIUM_PRIORITY', index=3, number=300,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGH_PRIORITY', index=4, number=400,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HIGHEST_PRIORITY', index=5, number=500,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=6199,
  serialized_end=6353,
)
_sym_db.RegisterEnumDescriptor(_OVERRIDECONFIGVALUE_OVERRIDEPRIORITY)


_RESPONSESTATUS = _descriptor.Descriptor(
  name='ResponseStatus',
  full_name='wireless_android_skyjam.ResponseStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response_code', full_name='wireless_android_skyjam.ResponseStatus.response_code', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RESPONSESTATUS_RESPONSECODE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=56,
  serialized_end=233,
)


_UPLOADOPERATION = _descriptor.Descriptor(
  name='UploadOperation',
  full_name='wireless_android_skyjam.UploadOperation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='wireless_android_skyjam.UploadOperation.operation', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPLOADOPERATION_OPERATION,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=236,
  serialized_end=403,
)


_ENHANCEDCHALLENGE = _descriptor.Descriptor(
  name='EnhancedChallenge',
  full_name='wireless_android_skyjam.EnhancedChallenge',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash_algorithm', full_name='wireless_android_skyjam.EnhancedChallenge.hash_algorithm', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='random_seed', full_name='wireless_android_skyjam.EnhancedChallenge.random_seed', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ENHANCEDCHALLENGE_HASHALGORITHM,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=406,
  serialized_end=567,
)


_CLIENTFPCHALLENGE = _descriptor.Descriptor(
  name='ClientFpChallenge',
  full_name='wireless_android_skyjam.ClientFpChallenge',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fingerprint_algorithm', full_name='wireless_android_skyjam.ClientFpChallenge.fingerprint_algorithm', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CLIENTFPCHALLENGE_FINGERPRINTALGORITHM,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=570,
  serialized_end=742,
)


_CHALLENGEINFO = _descriptor.Descriptor(
  name='ChallengeInfo',
  full_name='wireless_android_skyjam.ChallengeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_track_id', full_name='wireless_android_skyjam.ChallengeInfo.client_track_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_millis', full_name='wireless_android_skyjam.ChallengeInfo.start_millis', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='duration_millis', full_name='wireless_android_skyjam.ChallengeInfo.duration_millis', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='challenge_user_id', full_name='wireless_android_skyjam.ChallengeInfo.challenge_user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='challenge_timestamp', full_name='wireless_android_skyjam.ChallengeInfo.challenge_timestamp', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='expect_match', full_name='wireless_android_skyjam.ChallengeInfo.expect_match', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enhanced_challenge', full_name='wireless_android_skyjam.ChallengeInfo.enhanced_challenge', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='candidates', full_name='wireless_android_skyjam.ChallengeInfo.candidates', index=7,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='client_fp_challenge', full_name='wireless_android_skyjam.ChallengeInfo.client_fp_challenge', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=745,
  serialized_end=1075,
)


_ENHANCEDCHALLENGERESPONSE = _descriptor.Descriptor(
  name='EnhancedChallengeResponse',
  full_name='wireless_android_skyjam.EnhancedChallengeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='challenge_answer', full_name='wireless_android_skyjam.EnhancedChallengeResponse.challenge_answer', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1077,
  serialized_end=1130,
)


_SIGNEDCHALLENGEINFO = _descriptor.Descriptor(
  name='SignedChallengeInfo',
  full_name='wireless_android_skyjam.SignedChallengeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='challenge_info', full_name='wireless_android_skyjam.SignedChallengeInfo.challenge_info', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signature', full_name='wireless_android_skyjam.SignedChallengeInfo.signature', index=1,
      number=2, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1132,
  serialized_end=1236,
)


_TRACKSAMPLE = _descriptor.Descriptor(
  name='TrackSample',
  full_name='wireless_android_skyjam.TrackSample',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='track', full_name='wireless_android_skyjam.TrackSample.track', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signed_challenge_info', full_name='wireless_android_skyjam.TrackSample.signed_challenge_info', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sample', full_name='wireless_android_skyjam.TrackSample.sample', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sample_format', full_name='wireless_android_skyjam.TrackSample.sample_format', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_album_art', full_name='wireless_android_skyjam.TrackSample.user_album_art', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enhanced_challenge_response', full_name='wireless_android_skyjam.TrackSample.enhanced_challenge_response', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1239,
  serialized_end=1557,
)


_UPLOADPLAYLISTREQUEST = _descriptor.Descriptor(
  name='UploadPlaylistRequest',
  full_name='wireless_android_skyjam.UploadPlaylistRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='upload_operation', full_name='wireless_android_skyjam.UploadPlaylistRequest.upload_operation', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='playlist', full_name='wireless_android_skyjam.UploadPlaylistRequest.playlist', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UploadPlaylistRequest.uploader_id', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1560,
  serialized_end=1725,
)


_PLAYLISTRESPONSE = _descriptor.Descriptor(
  name='PlaylistResponse',
  full_name='wireless_android_skyjam.PlaylistResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response_status', full_name='wireless_android_skyjam.PlaylistResponse.response_status', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='client_id', full_name='wireless_android_skyjam.PlaylistResponse.client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='server_id', full_name='wireless_android_skyjam.PlaylistResponse.server_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1727,
  serialized_end=1849,
)


_UPLOADPLAYLISTRESPONSE = _descriptor.Descriptor(
  name='UploadPlaylistResponse',
  full_name='wireless_android_skyjam.UploadPlaylistResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='playlist_response', full_name='wireless_android_skyjam.UploadPlaylistResponse.playlist_response', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1851,
  serialized_end=1945,
)


_UPLOADPLAYLISTENTRYREQUEST = _descriptor.Descriptor(
  name='UploadPlaylistEntryRequest',
  full_name='wireless_android_skyjam.UploadPlaylistEntryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='upload_operation', full_name='wireless_android_skyjam.UploadPlaylistEntryRequest.upload_operation', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='playlist_entry', full_name='wireless_android_skyjam.UploadPlaylistEntryRequest.playlist_entry', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UploadPlaylistEntryRequest.uploader_id', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1948,
  serialized_end=2129,
)


_PLAYLISTENTRYRESPONSE = _descriptor.Descriptor(
  name='PlaylistEntryResponse',
  full_name='wireless_android_skyjam.PlaylistEntryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response_status', full_name='wireless_android_skyjam.PlaylistEntryResponse.response_status', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='client_id', full_name='wireless_android_skyjam.PlaylistEntryResponse.client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='server_id', full_name='wireless_android_skyjam.PlaylistEntryResponse.server_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2131,
  serialized_end=2258,
)


_UPLOADPLAYLISTENTRYRESPONSE = _descriptor.Descriptor(
  name='UploadPlaylistEntryResponse',
  full_name='wireless_android_skyjam.UploadPlaylistEntryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='playlist_entry_response', full_name='wireless_android_skyjam.UploadPlaylistEntryResponse.playlist_entry_response', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2260,
  serialized_end=2370,
)


_UPLOADMETADATAREQUEST = _descriptor.Descriptor(
  name='UploadMetadataRequest',
  full_name='wireless_android_skyjam.UploadMetadataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='track', full_name='wireless_android_skyjam.UploadMetadataRequest.track', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UploadMetadataRequest.uploader_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2372,
  serialized_end=2463,
)


_UPDATEUPLOADSTATEREQUEST = _descriptor.Descriptor(
  name='UpdateUploadStateRequest',
  full_name='wireless_android_skyjam.UpdateUploadStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='wireless_android_skyjam.UpdateUploadStateRequest.state', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UpdateUploadStateRequest.uploader_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPDATEUPLOADSTATEREQUEST_UPLOADSTATE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2466,
  serialized_end=2642,
)


_CLIENTSTATEREQUEST = _descriptor.Descriptor(
  name='ClientStateRequest',
  full_name='wireless_android_skyjam.ClientStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.ClientStateRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='get_purchased_tracks_since', full_name='wireless_android_skyjam.ClientStateRequest.get_purchased_tracks_since', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2644,
  serialized_end=2721,
)


_CLIENTSTATERESPONSE = _descriptor.Descriptor(
  name='ClientStateResponse',
  full_name='wireless_android_skyjam.ClientStateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='locker_track_limit', full_name='wireless_android_skyjam.ClientStateResponse.locker_track_limit', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_songs_in_locker', full_name='wireless_android_skyjam.ClientStateResponse.user_songs_in_locker', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='track_size_limit_in_mb', full_name='wireless_android_skyjam.ClientStateResponse.track_size_limit_in_mb', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_purchased_tracks_since', full_name='wireless_android_skyjam.ClientStateResponse.user_purchased_tracks_since', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_track_count', full_name='wireless_android_skyjam.ClientStateResponse.total_track_count', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='override_config', full_name='wireless_android_skyjam.ClientStateResponse.override_config', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2724,
  serialized_end=2980,
)


_UPLOADMETADATARESPONSE = _descriptor.Descriptor(
  name='UploadMetadataResponse',
  full_name='wireless_android_skyjam.UploadMetadataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='signed_challenge_info', full_name='wireless_android_skyjam.UploadMetadataResponse.signed_challenge_info', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='upload_id', full_name='wireless_android_skyjam.UploadMetadataResponse.upload_id', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='track_sample_response', full_name='wireless_android_skyjam.UploadMetadataResponse.track_sample_response', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2983,
  serialized_end=3180,
)


_TRACKSAMPLERESPONSE = _descriptor.Descriptor(
  name='TrackSampleResponse',
  full_name='wireless_android_skyjam.TrackSampleResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_track_id', full_name='wireless_android_skyjam.TrackSampleResponse.client_track_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='response_code', full_name='wireless_android_skyjam.TrackSampleResponse.response_code', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='server_track_id', full_name='wireless_android_skyjam.TrackSampleResponse.server_track_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='album_art_url', full_name='wireless_android_skyjam.TrackSampleResponse.album_art_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='expect_match', full_name='wireless_android_skyjam.TrackSampleResponse.expect_match', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TRACKSAMPLERESPONSE_RESPONSECODE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3183,
  serialized_end=3628,
)


_UPLOADSAMPLEREQUEST = _descriptor.Descriptor(
  name='UploadSampleRequest',
  full_name='wireless_android_skyjam.UploadSampleRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='track_sample', full_name='wireless_android_skyjam.UploadSampleRequest.track_sample', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UploadSampleRequest.uploader_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3630,
  serialized_end=3732,
)


_UPLOADSAMPLERESPONSE = _descriptor.Descriptor(
  name='UploadSampleResponse',
  full_name='wireless_android_skyjam.UploadSampleResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='track_sample_response', full_name='wireless_android_skyjam.UploadSampleResponse.track_sample_response', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3734,
  serialized_end=3833,
)


_IMAGEUNION = _descriptor.Descriptor(
  name='ImageUnion',
  full_name='wireless_android_skyjam.ImageUnion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_album_art', full_name='wireless_android_skyjam.ImageUnion.user_album_art', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='album_art_url', full_name='wireless_android_skyjam.ImageUnion.album_art_url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3835,
  serialized_end=3894,
)


_UPLOADRESPONSE = _descriptor.Descriptor(
  name='UploadResponse',
  full_name='wireless_android_skyjam.UploadResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response_type', full_name='wireless_android_skyjam.UploadResponse.response_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata_response', full_name='wireless_android_skyjam.UploadResponse.metadata_response', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='playlist_response', full_name='wireless_android_skyjam.UploadResponse.playlist_response', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='playlist_entry_response', full_name='wireless_android_skyjam.UploadResponse.playlist_entry_response', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sample_response', full_name='wireless_android_skyjam.UploadResponse.sample_response', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='getjobs_response', full_name='wireless_android_skyjam.UploadResponse.getjobs_response', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='clientstate_response', full_name='wireless_android_skyjam.UploadResponse.clientstate_response', index=6,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='policy', full_name='wireless_android_skyjam.UploadResponse.policy', index=7,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='auth_status', full_name='wireless_android_skyjam.UploadResponse.auth_status', index=8,
      number=11, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=8,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='auth_error', full_name='wireless_android_skyjam.UploadResponse.auth_error', index=9,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='upauth_response', full_name='wireless_android_skyjam.UploadResponse.upauth_response', index=10,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPLOADRESPONSE_RESPONSETYPE,
    _UPLOADRESPONSE_AUTHSTATUS,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3897,
  serialized_end=5149,
)


_TRACKSTOUPLOAD = _descriptor.Descriptor(
  name='TracksToUpload',
  full_name='wireless_android_skyjam.TracksToUpload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='wireless_android_skyjam.TracksToUpload.client_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='server_id', full_name='wireless_android_skyjam.TracksToUpload.server_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status', full_name='wireless_android_skyjam.TracksToUpload.status', index=2,
      number=5, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TRACKSTOUPLOAD_TRACKSTATUS,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5152,
  serialized_end=5332,
)


_GETJOBSREQUEST = _descriptor.Descriptor(
  name='GetJobsRequest',
  full_name='wireless_android_skyjam.GetJobsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.GetJobsRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5334,
  serialized_end=5371,
)


_GETJOBSRESPONSE = _descriptor.Descriptor(
  name='GetJobsResponse',
  full_name='wireless_android_skyjam.GetJobsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tracks_to_upload', full_name='wireless_android_skyjam.GetJobsResponse.tracks_to_upload', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='get_tracks_success', full_name='wireless_android_skyjam.GetJobsResponse.get_tracks_success', index=1,
      number=2, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5373,
  serialized_end=5485,
)


_UPAUTHREQUEST = _descriptor.Descriptor(
  name='UpAuthRequest',
  full_name='wireless_android_skyjam.UpAuthRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UpAuthRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='friendly_name', full_name='wireless_android_skyjam.UpAuthRequest.friendly_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mac_identifier', full_name='wireless_android_skyjam.UpAuthRequest.mac_identifier', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash_identifier', full_name='wireless_android_skyjam.UpAuthRequest.hash_identifier', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5487,
  serialized_end=5595,
)


_UPAUTHRESPONSE = _descriptor.Descriptor(
  name='UpAuthResponse',
  full_name='wireless_android_skyjam.UpAuthResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.UpAuthResponse.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5597,
  serialized_end=5634,
)


_DELETEUPLOADREQUESTEDREQUEST = _descriptor.Descriptor(
  name='DeleteUploadRequestedRequest',
  full_name='wireless_android_skyjam.DeleteUploadRequestedRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uploader_id', full_name='wireless_android_skyjam.DeleteUploadRequestedRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5636,
  serialized_end=5687,
)


_CLIENTPOLICY = _descriptor.Descriptor(
  name='ClientPolicy',
  full_name='wireless_android_skyjam.ClientPolicy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pause_uploads', full_name='wireless_android_skyjam.ClientPolicy.pause_uploads', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='abort', full_name='wireless_android_skyjam.ClientPolicy.abort', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='retry_in_minutes', full_name='wireless_android_skyjam.ClientPolicy.retry_in_minutes', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bandwidth_cap_kbps', full_name='wireless_android_skyjam.ClientPolicy.bandwidth_cap_kbps', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pause_downloads', full_name='wireless_android_skyjam.ClientPolicy.pause_downloads', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_bandwidth_cap_kbps', full_name='wireless_android_skyjam.ClientPolicy.download_bandwidth_cap_kbps', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='send_analytics', full_name='wireless_android_skyjam.ClientPolicy.send_analytics', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='analytics_url', full_name='wireless_android_skyjam.ClientPolicy.analytics_url', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enable_gapless', full_name='wireless_android_skyjam.ClientPolicy.enable_gapless', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enable_m4p', full_name='wireless_android_skyjam.ClientPolicy.enable_m4p', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5690,
  serialized_end=5949,
)


_UPLOADMETADATAINTERNALREQUEST = _descriptor.Descriptor(
  name='UploadMetadataInternalRequest',
  full_name='wireless_android_skyjam.UploadMetadataInternalRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='gaiaid', full_name='wireless_android_skyjam.UploadMetadataInternalRequest.gaiaid', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='request', full_name='wireless_android_skyjam.UploadMetadataInternalRequest.request', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5951,
  serialized_end=6063,
)


_OVERRIDECONFIGVALUE = _descriptor.Descriptor(
  name='OverrideConfigValue',
  full_name='wireless_android_skyjam.OverrideConfigValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='wireless_android_skyjam.OverrideConfigValue.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='wireless_android_skyjam.OverrideConfigValue.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='priority', full_name='wireless_android_skyjam.OverrideConfigValue.priority', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OVERRIDECONFIGVALUE_OVERRIDEPRIORITY,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6066,
  serialized_end=6353,
)


_OVERRIDECONFIGVALUECOLLECTION = _descriptor.Descriptor(
  name='OverrideConfigValueCollection',
  full_name='wireless_android_skyjam.OverrideConfigValueCollection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='wireless_android_skyjam.OverrideConfigValueCollection.value', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6355,
  serialized_end=6447,
)

_RESPONSESTATUS.fields_by_name['response_code'].enum_type = _RESPONSESTATUS_RESPONSECODE
_RESPONSESTATUS_RESPONSECODE.containing_type = _RESPONSESTATUS
_UPLOADOPERATION.fields_by_name['operation'].enum_type = _UPLOADOPERATION_OPERATION
_UPLOADOPERATION_OPERATION.containing_type = _UPLOADOPERATION
_ENHANCEDCHALLENGE.fields_by_name['hash_algorithm'].enum_type = _ENHANCEDCHALLENGE_HASHALGORITHM
_ENHANCEDCHALLENGE_HASHALGORITHM.containing_type = _ENHANCEDCHALLENGE
_CLIENTFPCHALLENGE.fields_by_name['fingerprint_algorithm'].enum_type = _CLIENTFPCHALLENGE_FINGERPRINTALGORITHM
_CLIENTFPCHALLENGE_FINGERPRINTALGORITHM.containing_type = _CLIENTFPCHALLENGE
_CHALLENGEINFO.fields_by_name['enhanced_challenge'].message_type = _ENHANCEDCHALLENGE
_CHALLENGEINFO.fields_by_name['client_fp_challenge'].message_type = _CLIENTFPCHALLENGE
_SIGNEDCHALLENGEINFO.fields_by_name['challenge_info'].message_type = _CHALLENGEINFO
_TRACKSAMPLE.fields_by_name['track'].message_type = locker__pb2._TRACK
_TRACKSAMPLE.fields_by_name['signed_challenge_info'].message_type = _SIGNEDCHALLENGEINFO
_TRACKSAMPLE.fields_by_name['sample_format'].enum_type = locker__pb2._TRACK_CONTENTTYPE
_TRACKSAMPLE.fields_by_name['user_album_art'].message_type = _IMAGEUNION
_UPLOADPLAYLISTREQUEST.fields_by_name['upload_operation'].message_type = _UPLOADOPERATION
_UPLOADPLAYLISTREQUEST.fields_by_name['playlist'].message_type = locker__pb2._PLAYLIST
_PLAYLISTRESPONSE.fields_by_name['response_status'].message_type = _RESPONSESTATUS
_UPLOADPLAYLISTRESPONSE.fields_by_name['playlist_response'].message_type = _PLAYLISTRESPONSE
_UPLOADPLAYLISTENTRYREQUEST.fields_by_name['upload_operation'].message_type = _UPLOADOPERATION
_UPLOADPLAYLISTENTRYREQUEST.fields_by_name['playlist_entry'].message_type = locker__pb2._PLAYLISTENTRY
_PLAYLISTENTRYRESPONSE.fields_by_name['response_status'].message_type = _RESPONSESTATUS
_UPLOADPLAYLISTENTRYRESPONSE.fields_by_name['playlist_entry_response'].message_type = _PLAYLISTENTRYRESPONSE
_UPLOADMETADATAREQUEST.fields_by_name['track'].message_type = locker__pb2._TRACK
_UPDATEUPLOADSTATEREQUEST.fields_by_name['state'].enum_type = _UPDATEUPLOADSTATEREQUEST_UPLOADSTATE
_UPDATEUPLOADSTATEREQUEST_UPLOADSTATE.containing_type = _UPDATEUPLOADSTATEREQUEST
_CLIENTSTATERESPONSE.fields_by_name['override_config'].message_type = _OVERRIDECONFIGVALUECOLLECTION
_UPLOADMETADATARESPONSE.fields_by_name['signed_challenge_info'].message_type = _SIGNEDCHALLENGEINFO
_UPLOADMETADATARESPONSE.fields_by_name['track_sample_response'].message_type = _TRACKSAMPLERESPONSE
_TRACKSAMPLERESPONSE.fields_by_name['response_code'].enum_type = _TRACKSAMPLERESPONSE_RESPONSECODE
_TRACKSAMPLERESPONSE_RESPONSECODE.containing_type = _TRACKSAMPLERESPONSE
_UPLOADSAMPLEREQUEST.fields_by_name['track_sample'].message_type = _TRACKSAMPLE
_UPLOADSAMPLERESPONSE.fields_by_name['track_sample_response'].message_type = _TRACKSAMPLERESPONSE
_UPLOADRESPONSE.fields_by_name['response_type'].enum_type = _UPLOADRESPONSE_RESPONSETYPE
_UPLOADRESPONSE.fields_by_name['metadata_response'].message_type = _UPLOADMETADATARESPONSE
_UPLOADRESPONSE.fields_by_name['playlist_response'].message_type = _UPLOADPLAYLISTRESPONSE
_UPLOADRESPONSE.fields_by_name['playlist_entry_response'].message_type = _UPLOADPLAYLISTENTRYRESPONSE
_UPLOADRESPONSE.fields_by_name['sample_response'].message_type = _UPLOADSAMPLERESPONSE
_UPLOADRESPONSE.fields_by_name['getjobs_response'].message_type = _GETJOBSRESPONSE
_UPLOADRESPONSE.fields_by_name['clientstate_response'].message_type = _CLIENTSTATERESPONSE
_UPLOADRESPONSE.fields_by_name['policy'].message_type = _CLIENTPOLICY
_UPLOADRESPONSE.fields_by_name['auth_status'].enum_type = _UPLOADRESPONSE_AUTHSTATUS
_UPLOADRESPONSE.fields_by_name['upauth_response'].message_type = _UPAUTHRESPONSE
_UPLOADRESPONSE_RESPONSETYPE.containing_type = _UPLOADRESPONSE
_UPLOADRESPONSE_AUTHSTATUS.containing_type = _UPLOADRESPONSE
_TRACKSTOUPLOAD.fields_by_name['status'].enum_type = _TRACKSTOUPLOAD_TRACKSTATUS
_TRACKSTOUPLOAD_TRACKSTATUS.containing_type = _TRACKSTOUPLOAD
_GETJOBSRESPONSE.fields_by_name['tracks_to_upload'].message_type = _TRACKSTOUPLOAD
_UPLOADMETADATAINTERNALREQUEST.fields_by_name['request'].message_type = _UPLOADMETADATAREQUEST
_OVERRIDECONFIGVALUE.fields_by_name['priority'].enum_type = _OVERRIDECONFIGVALUE_OVERRIDEPRIORITY
_OVERRIDECONFIGVALUE_OVERRIDEPRIORITY.containing_type = _OVERRIDECONFIGVALUE
_OVERRIDECONFIGVALUECOLLECTION.fields_by_name['value'].message_type = _OVERRIDECONFIGVALUE
DESCRIPTOR.message_types_by_name['ResponseStatus'] = _RESPONSESTATUS
DESCRIPTOR.message_types_by_name['UploadOperation'] = _UPLOADOPERATION
DESCRIPTOR.message_types_by_name['EnhancedChallenge'] = _ENHANCEDCHALLENGE
DESCRIPTOR.message_types_by_name['ClientFpChallenge'] = _CLIENTFPCHALLENGE
DESCRIPTOR.message_types_by_name['ChallengeInfo'] = _CHALLENGEINFO
DESCRIPTOR.message_types_by_name['EnhancedChallengeResponse'] = _ENHANCEDCHALLENGERESPONSE
DESCRIPTOR.message_types_by_name['SignedChallengeInfo'] = _SIGNEDCHALLENGEINFO
DESCRIPTOR.message_types_by_name['TrackSample'] = _TRACKSAMPLE
DESCRIPTOR.message_types_by_name['UploadPlaylistRequest'] = _UPLOADPLAYLISTREQUEST
DESCRIPTOR.message_types_by_name['PlaylistResponse'] = _PLAYLISTRESPONSE
DESCRIPTOR.message_types_by_name['UploadPlaylistResponse'] = _UPLOADPLAYLISTRESPONSE
DESCRIPTOR.message_types_by_name['UploadPlaylistEntryRequest'] = _UPLOADPLAYLISTENTRYREQUEST
DESCRIPTOR.message_types_by_name['PlaylistEntryResponse'] = _PLAYLISTENTRYRESPONSE
DESCRIPTOR.message_types_by_name['UploadPlaylistEntryResponse'] = _UPLOADPLAYLISTENTRYRESPONSE
DESCRIPTOR.message_types_by_name['UploadMetadataRequest'] = _UPLOADMETADATAREQUEST
DESCRIPTOR.message_types_by_name['UpdateUploadStateRequest'] = _UPDATEUPLOADSTATEREQUEST
DESCRIPTOR.message_types_by_name['ClientStateRequest'] = _CLIENTSTATEREQUEST
DESCRIPTOR.message_types_by_name['ClientStateResponse'] = _CLIENTSTATERESPONSE
DESCRIPTOR.message_types_by_name['UploadMetadataResponse'] = _UPLOADMETADATARESPONSE
DESCRIPTOR.message_types_by_name['TrackSampleResponse'] = _TRACKSAMPLERESPONSE
DESCRIPTOR.message_types_by_name['UploadSampleRequest'] = _UPLOADSAMPLEREQUEST
DESCRIPTOR.message_types_by_name['UploadSampleResponse'] = _UPLOADSAMPLERESPONSE
DESCRIPTOR.message_types_by_name['ImageUnion'] = _IMAGEUNION
DESCRIPTOR.message_types_by_name['UploadResponse'] = _UPLOADRESPONSE
DESCRIPTOR.message_types_by_name['TracksToUpload'] = _TRACKSTOUPLOAD
DESCRIPTOR.message_types_by_name['GetJobsRequest'] = _GETJOBSREQUEST
DESCRIPTOR.message_types_by_name['GetJobsResponse'] = _GETJOBSRESPONSE
DESCRIPTOR.message_types_by_name['UpAuthRequest'] = _UPAUTHREQUEST
DESCRIPTOR.message_types_by_name['UpAuthResponse'] = _UPAUTHRESPONSE
DESCRIPTOR.message_types_by_name['DeleteUploadRequestedRequest'] = _DELETEUPLOADREQUESTEDREQUEST
DESCRIPTOR.message_types_by_name['ClientPolicy'] = _CLIENTPOLICY
DESCRIPTOR.message_types_by_name['UploadMetadataInternalRequest'] = _UPLOADMETADATAINTERNALREQUEST
DESCRIPTOR.message_types_by_name['OverrideConfigValue'] = _OVERRIDECONFIGVALUE
DESCRIPTOR.message_types_by_name['OverrideConfigValueCollection'] = _OVERRIDECONFIGVALUECOLLECTION

ResponseStatus = _reflection.GeneratedProtocolMessageType('ResponseStatus', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSESTATUS,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ResponseStatus)
  ))
_sym_db.RegisterMessage(ResponseStatus)

UploadOperation = _reflection.GeneratedProtocolMessageType('UploadOperation', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADOPERATION,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadOperation)
  ))
_sym_db.RegisterMessage(UploadOperation)

EnhancedChallenge = _reflection.GeneratedProtocolMessageType('EnhancedChallenge', (_message.Message,), dict(
  DESCRIPTOR = _ENHANCEDCHALLENGE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.EnhancedChallenge)
  ))
_sym_db.RegisterMessage(EnhancedChallenge)

ClientFpChallenge = _reflection.GeneratedProtocolMessageType('ClientFpChallenge', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTFPCHALLENGE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ClientFpChallenge)
  ))
_sym_db.RegisterMessage(ClientFpChallenge)

ChallengeInfo = _reflection.GeneratedProtocolMessageType('ChallengeInfo', (_message.Message,), dict(
  DESCRIPTOR = _CHALLENGEINFO,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ChallengeInfo)
  ))
_sym_db.RegisterMessage(ChallengeInfo)

EnhancedChallengeResponse = _reflection.GeneratedProtocolMessageType('EnhancedChallengeResponse', (_message.Message,), dict(
  DESCRIPTOR = _ENHANCEDCHALLENGERESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.EnhancedChallengeResponse)
  ))
_sym_db.RegisterMessage(EnhancedChallengeResponse)

SignedChallengeInfo = _reflection.GeneratedProtocolMessageType('SignedChallengeInfo', (_message.Message,), dict(
  DESCRIPTOR = _SIGNEDCHALLENGEINFO,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.SignedChallengeInfo)
  ))
_sym_db.RegisterMessage(SignedChallengeInfo)

TrackSample = _reflection.GeneratedProtocolMessageType('TrackSample', (_message.Message,), dict(
  DESCRIPTOR = _TRACKSAMPLE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.TrackSample)
  ))
_sym_db.RegisterMessage(TrackSample)

UploadPlaylistRequest = _reflection.GeneratedProtocolMessageType('UploadPlaylistRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADPLAYLISTREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadPlaylistRequest)
  ))
_sym_db.RegisterMessage(UploadPlaylistRequest)

PlaylistResponse = _reflection.GeneratedProtocolMessageType('PlaylistResponse', (_message.Message,), dict(
  DESCRIPTOR = _PLAYLISTRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.PlaylistResponse)
  ))
_sym_db.RegisterMessage(PlaylistResponse)

UploadPlaylistResponse = _reflection.GeneratedProtocolMessageType('UploadPlaylistResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADPLAYLISTRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadPlaylistResponse)
  ))
_sym_db.RegisterMessage(UploadPlaylistResponse)

UploadPlaylistEntryRequest = _reflection.GeneratedProtocolMessageType('UploadPlaylistEntryRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADPLAYLISTENTRYREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadPlaylistEntryRequest)
  ))
_sym_db.RegisterMessage(UploadPlaylistEntryRequest)

PlaylistEntryResponse = _reflection.GeneratedProtocolMessageType('PlaylistEntryResponse', (_message.Message,), dict(
  DESCRIPTOR = _PLAYLISTENTRYRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.PlaylistEntryResponse)
  ))
_sym_db.RegisterMessage(PlaylistEntryResponse)

UploadPlaylistEntryResponse = _reflection.GeneratedProtocolMessageType('UploadPlaylistEntryResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADPLAYLISTENTRYRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadPlaylistEntryResponse)
  ))
_sym_db.RegisterMessage(UploadPlaylistEntryResponse)

UploadMetadataRequest = _reflection.GeneratedProtocolMessageType('UploadMetadataRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADMETADATAREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadMetadataRequest)
  ))
_sym_db.RegisterMessage(UploadMetadataRequest)

UpdateUploadStateRequest = _reflection.GeneratedProtocolMessageType('UpdateUploadStateRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPDATEUPLOADSTATEREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UpdateUploadStateRequest)
  ))
_sym_db.RegisterMessage(UpdateUploadStateRequest)

ClientStateRequest = _reflection.GeneratedProtocolMessageType('ClientStateRequest', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTSTATEREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ClientStateRequest)
  ))
_sym_db.RegisterMessage(ClientStateRequest)

ClientStateResponse = _reflection.GeneratedProtocolMessageType('ClientStateResponse', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTSTATERESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ClientStateResponse)
  ))
_sym_db.RegisterMessage(ClientStateResponse)

UploadMetadataResponse = _reflection.GeneratedProtocolMessageType('UploadMetadataResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADMETADATARESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadMetadataResponse)
  ))
_sym_db.RegisterMessage(UploadMetadataResponse)

TrackSampleResponse = _reflection.GeneratedProtocolMessageType('TrackSampleResponse', (_message.Message,), dict(
  DESCRIPTOR = _TRACKSAMPLERESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.TrackSampleResponse)
  ))
_sym_db.RegisterMessage(TrackSampleResponse)

UploadSampleRequest = _reflection.GeneratedProtocolMessageType('UploadSampleRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADSAMPLEREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadSampleRequest)
  ))
_sym_db.RegisterMessage(UploadSampleRequest)

UploadSampleResponse = _reflection.GeneratedProtocolMessageType('UploadSampleResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADSAMPLERESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadSampleResponse)
  ))
_sym_db.RegisterMessage(UploadSampleResponse)

ImageUnion = _reflection.GeneratedProtocolMessageType('ImageUnion', (_message.Message,), dict(
  DESCRIPTOR = _IMAGEUNION,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ImageUnion)
  ))
_sym_db.RegisterMessage(ImageUnion)

UploadResponse = _reflection.GeneratedProtocolMessageType('UploadResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadResponse)
  ))
_sym_db.RegisterMessage(UploadResponse)

TracksToUpload = _reflection.GeneratedProtocolMessageType('TracksToUpload', (_message.Message,), dict(
  DESCRIPTOR = _TRACKSTOUPLOAD,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.TracksToUpload)
  ))
_sym_db.RegisterMessage(TracksToUpload)

GetJobsRequest = _reflection.GeneratedProtocolMessageType('GetJobsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETJOBSREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.GetJobsRequest)
  ))
_sym_db.RegisterMessage(GetJobsRequest)

GetJobsResponse = _reflection.GeneratedProtocolMessageType('GetJobsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETJOBSRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.GetJobsResponse)
  ))
_sym_db.RegisterMessage(GetJobsResponse)

UpAuthRequest = _reflection.GeneratedProtocolMessageType('UpAuthRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPAUTHREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UpAuthRequest)
  ))
_sym_db.RegisterMessage(UpAuthRequest)

UpAuthResponse = _reflection.GeneratedProtocolMessageType('UpAuthResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPAUTHRESPONSE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UpAuthResponse)
  ))
_sym_db.RegisterMessage(UpAuthResponse)

DeleteUploadRequestedRequest = _reflection.GeneratedProtocolMessageType('DeleteUploadRequestedRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETEUPLOADREQUESTEDREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.DeleteUploadRequestedRequest)
  ))
_sym_db.RegisterMessage(DeleteUploadRequestedRequest)

ClientPolicy = _reflection.GeneratedProtocolMessageType('ClientPolicy', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTPOLICY,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ClientPolicy)
  ))
_sym_db.RegisterMessage(ClientPolicy)

UploadMetadataInternalRequest = _reflection.GeneratedProtocolMessageType('UploadMetadataInternalRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADMETADATAINTERNALREQUEST,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UploadMetadataInternalRequest)
  ))
_sym_db.RegisterMessage(UploadMetadataInternalRequest)

OverrideConfigValue = _reflection.GeneratedProtocolMessageType('OverrideConfigValue', (_message.Message,), dict(
  DESCRIPTOR = _OVERRIDECONFIGVALUE,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.OverrideConfigValue)
  ))
_sym_db.RegisterMessage(OverrideConfigValue)

OverrideConfigValueCollection = _reflection.GeneratedProtocolMessageType('OverrideConfigValueCollection', (_message.Message,), dict(
  DESCRIPTOR = _OVERRIDECONFIGVALUECOLLECTION,
  __module__ = 'upload_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.OverrideConfigValueCollection)
  ))
_sym_db.RegisterMessage(OverrideConfigValueCollection)


# @@protoc_insertion_point(module_scope)
