# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: uits.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='uits.proto',
  package='wireless_android_skyjam',
  syntax='proto2',
  serialized_pb=_b('\n\nuits.proto\x12\x17wireless_android_skyjam\"\x83\x01\n\tProductId\x12\x35\n\x04type\x18\x01 \x02(\x0e\x32\'.wireless_android_skyjam.ProductId.Type\x12\x18\n\tcompleted\x18\x02 \x01(\x08:\x05\x66\x61lse\x12\n\n\x02id\x18\x03 \x02(\t\"\x19\n\x04Type\x12\x07\n\x03UPC\x10\x01\x12\x08\n\x04GRID\x10\x02\"\\\n\x07\x41ssetId\x12\x33\n\x04type\x18\x01 \x02(\x0e\x32%.wireless_android_skyjam.AssetId.Type\x12\n\n\x02id\x18\x02 \x02(\t\"\x10\n\x04Type\x12\x08\n\x04ISRC\x10\x01\"/\n\rTransactionId\x12\x12\n\x07version\x18\x01 \x02(\t:\x01\x31\x12\n\n\x02id\x18\x02 \x02(\t\"|\n\x07MediaId\x12\x46\n\x0e\x61lgorithm_type\x18\x01 \x02(\x0e\x32..wireless_android_skyjam.MediaId.AlgorithmType\x12\x0c\n\x04hash\x18\x02 \x02(\t\"\x1b\n\rAlgorithmType\x12\n\n\x06SHA256\x10\x01\"\xa3\x01\n\x07UrlInfo\x12\x33\n\x04type\x18\x01 \x02(\x0e\x32%.wireless_android_skyjam.UrlInfo.Type\x12\x0b\n\x03url\x18\x02 \x02(\t\"V\n\x04Type\x12\x08\n\x04WCOM\x10\x01\x12\x08\n\x04WCOP\x10\x02\x12\x08\n\x04WOAF\x10\x03\x12\x08\n\x04WOAR\x10\x04\x12\x08\n\x04WOAS\x10\x05\x12\x08\n\x04WORS\x10\x06\x12\x08\n\x04WPAY\x10\x07\x12\x08\n\x04WPUB\x10\x08\"\xac\x01\n\x0f\x43opyrightStatus\x12;\n\x04type\x18\x01 \x02(\x0e\x32-.wireless_android_skyjam.CopyrightStatus.Type\x12\x11\n\tcopyright\x18\x02 \x01(\t\"I\n\x04Type\x12\x0f\n\x0bUNSPECIFIED\x10\x01\x12\x15\n\x11\x41LLRIGHTSRESERVED\x10\x02\x12\x0e\n\nPRERELEASE\x10\x03\x12\t\n\x05OTHER\x10\x04\"$\n\x05\x45xtra\x12\x0c\n\x04type\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"\xf5\x04\n\x0cUitsMetadata\x12\r\n\x05nonce\x18\x01 \x02(\t\x12\x16\n\x0e\x64istributor_id\x18\x02 \x02(\t\x12\x18\n\x10transaction_date\x18\x03 \x02(\t\x12\x36\n\nproduct_id\x18\x04 \x02(\x0b\x32\".wireless_android_skyjam.ProductId\x12\x32\n\x08\x61sset_id\x18\x05 \x02(\x0b\x32 .wireless_android_skyjam.AssetId\x12>\n\x0etransaction_id\x18\x06 \x02(\x0b\x32&.wireless_android_skyjam.TransactionId\x12\x32\n\x08media_id\x18\x07 \x02(\x0b\x32 .wireless_android_skyjam.MediaId\x12\x32\n\x08url_info\x18\x08 \x01(\x0b\x32 .wireless_android_skyjam.UrlInfo\x12Z\n\x16parental_advisory_type\x18\t \x01(\x0e\x32:.wireless_android_skyjam.UitsMetadata.ParentalAdvisoryType\x12\x42\n\x10\x63opyright_status\x18\n \x01(\x0b\x32(.wireless_android_skyjam.CopyrightStatus\x12-\n\x05\x65xtra\x18\x0b \x03(\x0b\x32\x1e.wireless_android_skyjam.Extra\"A\n\x14ParentalAdvisoryType\x12\x0f\n\x0bUNSPECIFIED\x10\x01\x12\x0c\n\x08\x45XPLICIT\x10\x02\x12\n\n\x06\x45\x44ITED\x10\x03\"\xa5\x02\n\rUitsSignature\x12L\n\x0e\x61lgorithm_type\x18\x01 \x02(\x0e\x32\x34.wireless_android_skyjam.UitsSignature.AlgorithmType\x12Z\n\x15\x63\x61nonicalization_type\x18\x02 \x02(\x0e\x32;.wireless_android_skyjam.UitsSignature.CanonicalizationType\x12\x0e\n\x06key_id\x18\x03 \x02(\t\x12\r\n\x05value\x18\x04 \x02(\t\")\n\rAlgorithmType\x12\x0b\n\x07RSA2048\x10\x01\x12\x0b\n\x07\x44SA2048\x10\x02\" \n\x14\x43\x61nonicalizationType\x12\x08\n\x04NONE\x10\x01\"z\n\x04Uits\x12\x37\n\x08metadata\x18\x01 \x02(\x0b\x32%.wireless_android_skyjam.UitsMetadata\x12\x39\n\tsignature\x18\x02 \x02(\x0b\x32&.wireless_android_skyjam.UitsSignature')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_PRODUCTID_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='wireless_android_skyjam.ProductId.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UPC', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GRID', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=146,
  serialized_end=171,
)
_sym_db.RegisterEnumDescriptor(_PRODUCTID_TYPE)

_ASSETID_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='wireless_android_skyjam.AssetId.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ISRC', index=0, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=249,
  serialized_end=265,
)
_sym_db.RegisterEnumDescriptor(_ASSETID_TYPE)

_MEDIAID_ALGORITHMTYPE = _descriptor.EnumDescriptor(
  name='AlgorithmType',
  full_name='wireless_android_skyjam.MediaId.AlgorithmType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SHA256', index=0, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=413,
  serialized_end=440,
)
_sym_db.RegisterEnumDescriptor(_MEDIAID_ALGORITHMTYPE)

_URLINFO_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='wireless_android_skyjam.UrlInfo.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WCOM', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WCOP', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WOAF', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WOAR', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WOAS', index=4, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WORS', index=5, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WPAY', index=6, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WPUB', index=7, number=8,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=520,
  serialized_end=606,
)
_sym_db.RegisterEnumDescriptor(_URLINFO_TYPE)

_COPYRIGHTSTATUS_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='wireless_android_skyjam.CopyrightStatus.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ALLRIGHTSRESERVED', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PRERELEASE', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OTHER', index=3, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=708,
  serialized_end=781,
)
_sym_db.RegisterEnumDescriptor(_COPYRIGHTSTATUS_TYPE)

_UITSMETADATA_PARENTALADVISORYTYPE = _descriptor.EnumDescriptor(
  name='ParentalAdvisoryType',
  full_name='wireless_android_skyjam.UitsMetadata.ParentalAdvisoryType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EXPLICIT', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EDITED', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1386,
  serialized_end=1451,
)
_sym_db.RegisterEnumDescriptor(_UITSMETADATA_PARENTALADVISORYTYPE)

_UITSSIGNATURE_ALGORITHMTYPE = _descriptor.EnumDescriptor(
  name='AlgorithmType',
  full_name='wireless_android_skyjam.UitsSignature.AlgorithmType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RSA2048', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DSA2048', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1672,
  serialized_end=1713,
)
_sym_db.RegisterEnumDescriptor(_UITSSIGNATURE_ALGORITHMTYPE)

_UITSSIGNATURE_CANONICALIZATIONTYPE = _descriptor.EnumDescriptor(
  name='CanonicalizationType',
  full_name='wireless_android_skyjam.UitsSignature.CanonicalizationType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1715,
  serialized_end=1747,
)
_sym_db.RegisterEnumDescriptor(_UITSSIGNATURE_CANONICALIZATIONTYPE)


_PRODUCTID = _descriptor.Descriptor(
  name='ProductId',
  full_name='wireless_android_skyjam.ProductId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='wireless_android_skyjam.ProductId.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='completed', full_name='wireless_android_skyjam.ProductId.completed', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='wireless_android_skyjam.ProductId.id', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PRODUCTID_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=171,
)


_ASSETID = _descriptor.Descriptor(
  name='AssetId',
  full_name='wireless_android_skyjam.AssetId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='wireless_android_skyjam.AssetId.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='wireless_android_skyjam.AssetId.id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ASSETID_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=173,
  serialized_end=265,
)


_TRANSACTIONID = _descriptor.Descriptor(
  name='TransactionId',
  full_name='wireless_android_skyjam.TransactionId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='wireless_android_skyjam.TransactionId.version', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=True, default_value=_b("1").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='wireless_android_skyjam.TransactionId.id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=267,
  serialized_end=314,
)


_MEDIAID = _descriptor.Descriptor(
  name='MediaId',
  full_name='wireless_android_skyjam.MediaId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='algorithm_type', full_name='wireless_android_skyjam.MediaId.algorithm_type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='wireless_android_skyjam.MediaId.hash', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _MEDIAID_ALGORITHMTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=316,
  serialized_end=440,
)


_URLINFO = _descriptor.Descriptor(
  name='UrlInfo',
  full_name='wireless_android_skyjam.UrlInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='wireless_android_skyjam.UrlInfo.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='url', full_name='wireless_android_skyjam.UrlInfo.url', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _URLINFO_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=606,
)


_COPYRIGHTSTATUS = _descriptor.Descriptor(
  name='CopyrightStatus',
  full_name='wireless_android_skyjam.CopyrightStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='wireless_android_skyjam.CopyrightStatus.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='copyright', full_name='wireless_android_skyjam.CopyrightStatus.copyright', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _COPYRIGHTSTATUS_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=609,
  serialized_end=781,
)


_EXTRA = _descriptor.Descriptor(
  name='Extra',
  full_name='wireless_android_skyjam.Extra',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='wireless_android_skyjam.Extra.type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='wireless_android_skyjam.Extra.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=783,
  serialized_end=819,
)


_UITSMETADATA = _descriptor.Descriptor(
  name='UitsMetadata',
  full_name='wireless_android_skyjam.UitsMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nonce', full_name='wireless_android_skyjam.UitsMetadata.nonce', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='distributor_id', full_name='wireless_android_skyjam.UitsMetadata.distributor_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='transaction_date', full_name='wireless_android_skyjam.UitsMetadata.transaction_date', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='product_id', full_name='wireless_android_skyjam.UitsMetadata.product_id', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='asset_id', full_name='wireless_android_skyjam.UitsMetadata.asset_id', index=4,
      number=5, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='transaction_id', full_name='wireless_android_skyjam.UitsMetadata.transaction_id', index=5,
      number=6, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='media_id', full_name='wireless_android_skyjam.UitsMetadata.media_id', index=6,
      number=7, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='url_info', full_name='wireless_android_skyjam.UitsMetadata.url_info', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parental_advisory_type', full_name='wireless_android_skyjam.UitsMetadata.parental_advisory_type', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='copyright_status', full_name='wireless_android_skyjam.UitsMetadata.copyright_status', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='extra', full_name='wireless_android_skyjam.UitsMetadata.extra', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UITSMETADATA_PARENTALADVISORYTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=822,
  serialized_end=1451,
)


_UITSSIGNATURE = _descriptor.Descriptor(
  name='UitsSignature',
  full_name='wireless_android_skyjam.UitsSignature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='algorithm_type', full_name='wireless_android_skyjam.UitsSignature.algorithm_type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='canonicalization_type', full_name='wireless_android_skyjam.UitsSignature.canonicalization_type', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key_id', full_name='wireless_android_skyjam.UitsSignature.key_id', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='wireless_android_skyjam.UitsSignature.value', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UITSSIGNATURE_ALGORITHMTYPE,
    _UITSSIGNATURE_CANONICALIZATIONTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1454,
  serialized_end=1747,
)


_UITS = _descriptor.Descriptor(
  name='Uits',
  full_name='wireless_android_skyjam.Uits',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='wireless_android_skyjam.Uits.metadata', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signature', full_name='wireless_android_skyjam.Uits.signature', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1749,
  serialized_end=1871,
)

_PRODUCTID.fields_by_name['type'].enum_type = _PRODUCTID_TYPE
_PRODUCTID_TYPE.containing_type = _PRODUCTID
_ASSETID.fields_by_name['type'].enum_type = _ASSETID_TYPE
_ASSETID_TYPE.containing_type = _ASSETID
_MEDIAID.fields_by_name['algorithm_type'].enum_type = _MEDIAID_ALGORITHMTYPE
_MEDIAID_ALGORITHMTYPE.containing_type = _MEDIAID
_URLINFO.fields_by_name['type'].enum_type = _URLINFO_TYPE
_URLINFO_TYPE.containing_type = _URLINFO
_COPYRIGHTSTATUS.fields_by_name['type'].enum_type = _COPYRIGHTSTATUS_TYPE
_COPYRIGHTSTATUS_TYPE.containing_type = _COPYRIGHTSTATUS
_UITSMETADATA.fields_by_name['product_id'].message_type = _PRODUCTID
_UITSMETADATA.fields_by_name['asset_id'].message_type = _ASSETID
_UITSMETADATA.fields_by_name['transaction_id'].message_type = _TRANSACTIONID
_UITSMETADATA.fields_by_name['media_id'].message_type = _MEDIAID
_UITSMETADATA.fields_by_name['url_info'].message_type = _URLINFO
_UITSMETADATA.fields_by_name['parental_advisory_type'].enum_type = _UITSMETADATA_PARENTALADVISORYTYPE
_UITSMETADATA.fields_by_name['copyright_status'].message_type = _COPYRIGHTSTATUS
_UITSMETADATA.fields_by_name['extra'].message_type = _EXTRA
_UITSMETADATA_PARENTALADVISORYTYPE.containing_type = _UITSMETADATA
_UITSSIGNATURE.fields_by_name['algorithm_type'].enum_type = _UITSSIGNATURE_ALGORITHMTYPE
_UITSSIGNATURE.fields_by_name['canonicalization_type'].enum_type = _UITSSIGNATURE_CANONICALIZATIONTYPE
_UITSSIGNATURE_ALGORITHMTYPE.containing_type = _UITSSIGNATURE
_UITSSIGNATURE_CANONICALIZATIONTYPE.containing_type = _UITSSIGNATURE
_UITS.fields_by_name['metadata'].message_type = _UITSMETADATA
_UITS.fields_by_name['signature'].message_type = _UITSSIGNATURE
DESCRIPTOR.message_types_by_name['ProductId'] = _PRODUCTID
DESCRIPTOR.message_types_by_name['AssetId'] = _ASSETID
DESCRIPTOR.message_types_by_name['TransactionId'] = _TRANSACTIONID
DESCRIPTOR.message_types_by_name['MediaId'] = _MEDIAID
DESCRIPTOR.message_types_by_name['UrlInfo'] = _URLINFO
DESCRIPTOR.message_types_by_name['CopyrightStatus'] = _COPYRIGHTSTATUS
DESCRIPTOR.message_types_by_name['Extra'] = _EXTRA
DESCRIPTOR.message_types_by_name['UitsMetadata'] = _UITSMETADATA
DESCRIPTOR.message_types_by_name['UitsSignature'] = _UITSSIGNATURE
DESCRIPTOR.message_types_by_name['Uits'] = _UITS

ProductId = _reflection.GeneratedProtocolMessageType('ProductId', (_message.Message,), dict(
  DESCRIPTOR = _PRODUCTID,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.ProductId)
  ))
_sym_db.RegisterMessage(ProductId)

AssetId = _reflection.GeneratedProtocolMessageType('AssetId', (_message.Message,), dict(
  DESCRIPTOR = _ASSETID,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.AssetId)
  ))
_sym_db.RegisterMessage(AssetId)

TransactionId = _reflection.GeneratedProtocolMessageType('TransactionId', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTIONID,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.TransactionId)
  ))
_sym_db.RegisterMessage(TransactionId)

MediaId = _reflection.GeneratedProtocolMessageType('MediaId', (_message.Message,), dict(
  DESCRIPTOR = _MEDIAID,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.MediaId)
  ))
_sym_db.RegisterMessage(MediaId)

UrlInfo = _reflection.GeneratedProtocolMessageType('UrlInfo', (_message.Message,), dict(
  DESCRIPTOR = _URLINFO,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UrlInfo)
  ))
_sym_db.RegisterMessage(UrlInfo)

CopyrightStatus = _reflection.GeneratedProtocolMessageType('CopyrightStatus', (_message.Message,), dict(
  DESCRIPTOR = _COPYRIGHTSTATUS,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.CopyrightStatus)
  ))
_sym_db.RegisterMessage(CopyrightStatus)

Extra = _reflection.GeneratedProtocolMessageType('Extra', (_message.Message,), dict(
  DESCRIPTOR = _EXTRA,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.Extra)
  ))
_sym_db.RegisterMessage(Extra)

UitsMetadata = _reflection.GeneratedProtocolMessageType('UitsMetadata', (_message.Message,), dict(
  DESCRIPTOR = _UITSMETADATA,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UitsMetadata)
  ))
_sym_db.RegisterMessage(UitsMetadata)

UitsSignature = _reflection.GeneratedProtocolMessageType('UitsSignature', (_message.Message,), dict(
  DESCRIPTOR = _UITSSIGNATURE,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.UitsSignature)
  ))
_sym_db.RegisterMessage(UitsSignature)

Uits = _reflection.GeneratedProtocolMessageType('Uits', (_message.Message,), dict(
  DESCRIPTOR = _UITS,
  __module__ = 'uits_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.Uits)
  ))
_sym_db.RegisterMessage(Uits)


# @@protoc_insertion_point(module_scope)
