# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import, unicode_literals
from gmusicapi.clients.webclient import Webclient
from gmusicapi.clients.musicmanager import Musicmanager, OAUTH_FILEPATH
from gmusicapi.clients.mobileclient import Mobileclient

(Webclient, Musicmanager, Mobileclient, OAUTH_FILEPATH)  # noqa
